package com.brilliance.eibs.server;

import com.brilliance.eibs.core.exception.InterfaceException;
import com.brilliance.eibs.core.model.IServiceDef;
import com.brilliance.eibs.core.service.Context;
import com.brilliance.eibs.el.ApacheEL;
import com.brilliance.eibs.server.channel.server.impl.AsyncChannelServer;
import com.brilliance.eibs.server.channel.server.impl.AsyncFixedChannelServer;
import com.brilliance.eibs.server.connection.Detect;
import com.brilliance.eibs.util.StringUtil;

import java.io.IOException;
import java.net.SocketException;

public class ChannelServer extends AbsServer {

	private AsyncChannelServer server;
	private ChannelListner listener;

	public ChannelServer(Context context, IServiceDef serviceDef, ApacheEL elParser) {
		super(context, serviceDef, elParser);
	}

	@Override
	public void run() {
		logger.info( LOG_FLAG + "ChannelServer starting ." + LOG_FLAG);
		int port = Integer.valueOf(getRequiredPropertyValue("port"));
		int destPort = Integer.valueOf(getRequiredPropertyValue("destPort"));
		String destIP = getRequiredPropertyValue("destIP");
		String requestInterfaceName = getPropertyValue("request_interfaceName");
		String requestTransactionId = getPropertyValue("request_transactionName");
		String responseInterfaceName = getPropertyValue("response_interfaceName");
		String responseTransactionId = getPropertyValue("response_transactionName");
		String length = getPropertyValue("length");
		if (StringUtil.isEmpty(length)
				|| ((StringUtil.isEmpty(requestInterfaceName) || StringUtil.isEmpty(requestTransactionId)) && (StringUtil
						.isEmpty(responseInterfaceName) || StringUtil.isEmpty(responseTransactionId))))
			server = new AsyncChannelServer(port, destPort, destIP);
		else
			server = new AsyncFixedChannelServer(port, destPort, destIP, Integer.valueOf(length), requestInterfaceName, requestTransactionId,
					responseInterfaceName, responseTransactionId);

		listener = new ChannelListner();
		Thread t = new Thread(listener);
		t.setName("Asynchronous Channel '" + serviceDef.getId() + "' Server Listener");
		t.setDaemon(true);
		t.start();

		try {
			server.startup();
		} catch (IOException e) {
			if ((e instanceof SocketException) && e.getMessage().equals("socket closed"))
				;
			else {
				throw new InterfaceException("02301", e);
			}
		}
		logger.info( LOG_FLAG + "ChannelServer is finished." + LOG_FLAG);
	}

	@Override
	public void close() {
		listener.interrupted();
		server.close();
	}

	private class ChannelListner extends Detect {
		private long interval = 60000;

		private ChannelListner() {
		}

		private ChannelListner(long interval) {
			this.interval = interval;
		}

		@Override
		protected void exception() {
			interrupted();
		}

		@Override
		protected void handle() throws IOException {
			int size = server.getPool().size();
			logger.info( "Connections size in [" + serviceDef.getId() + "] channel : " + size);
			try {
				Thread.sleep(interval);
			} catch (InterruptedException e) {
				logger.error( e.getMessage(),e);
			}
		}

		@Override
		protected void close() {
			logger.info( "ChannelListner of [" + serviceDef.getId() + "] channel is closed.");
		}

	}

}
