package com.brilliance.eibs.server;

import com.brilliance.eibs.core.frame.SocketProxyFactry;
import com.brilliance.eibs.core.model.IServiceDef;
import com.brilliance.eibs.core.service.Context;
import com.brilliance.eibs.el.ApacheEL;
import com.brilliance.eibs.server.socket.IServerMode;
import com.brilliance.eibs.server.socket.impl.MinaNioMode;
import com.brilliance.eibs.server.socket.impl.SocketServerMode;
import com.brilliance.eibs.util.LogUtil;

/**
 * socket服务
 *
 * @author xiaoyuanzhen ------------modified by weicong on 2016/4/01 -----------
 */
public class SocketServer extends AbsServer {

    private int port;
    private int queueLen;
    private int size;
    private String delayTime;
    private String interfaceName;
    private String transName;
    private int timeout;
    private IServerMode mode;

    public SocketServer(Context context, IServiceDef serviceDef, ApacheEL elParser) {
        super(context, serviceDef, elParser);
    }

    @Override
    public void run() {
        logger.info(LOG_FLAG + "SocketServer starting ." + LOG_FLAG);
        // 服务发布的端口
        port = Integer.valueOf(getRequiredPropertyValue(PORT_SYMBOL));
        logger.info("socketserver port=" + port);
        // 等待队列的长度
        queueLen = Integer.valueOf(getRequiredPropertyValue(REQUEST_QUEUE_LEN_SYMBOL));
        size = Integer.valueOf(getRequiredPropertyValue(THREAD_POOL_SIZE_SYMBOL));

        String trustips = getPropertyValue(TRUST_IP_SYMBOL);
        // 延迟启动时间
        delayTime = getPropertyValue(DELAY_TIME);
        String type = getPropertyValue(TYPE);
        interfaceName = getRequiredPropertyValue("interfaceName");
        transName = getRequiredPropertyValue("transactionName");
        timeout = Integer.valueOf(getRequiredPropertyValue(RECEIVE_TIMEOUT_SYMBOL));

        if (SocketProxyFactry.MINA_TYPE.equals(type)) {
            mode = new MinaNioMode.Builder(port, queueLen).setProps(serviceDef.getPropertyMap()).setInterfaceName(interfaceName)
                    .setServerId(serviceDef.getId()).setTimeout(timeout).setTransName(transName).setTrustips(trustips)
                    .setLogger(LogUtil.getLogger(context)).build();
        } else
            /*
             * mode = new SocketServerMode.Builder(port, queueLen,
             * size).setArgs(args)
             * .setDelayTime(delayTime).setExpression(serviceDef.getExpression())
             * .setInterfaceName(interfaceName).setServerId(serviceDef.getId())
             * .setTimeout(timeout).setTransName(transName)
             * .setTrustips(trustips).setLogger
             * (BeaLogFactory.getBeaLog(context)).build();
             */ {
            mode = new SocketServerMode(SocketProxyFactry.ORG_SOCKET_TYPE, trustips, port, queueLen, size, delayTime, interfaceName, transName,
                    timeout, serviceDef.getExpression(), args, serviceDef.getId(), LogUtil.getLogger(context));
        }
        mode.startup();
        logger.info(LOG_FLAG + "SocketServer is finished." + LOG_FLAG);
    }

    /**
     * 关闭ServerSocket
     */
    @Override
    public void close() {
        mode.close();
    }

}
