package com.brilliance.eibs.server;

import com.brilliance.eibs.bean.ParseConfig;
import com.brilliance.eibs.core.exception.InterfaceException;
import com.brilliance.eibs.core.model.IServiceDef;
import com.brilliance.eibs.core.service.Context;
import com.brilliance.eibs.core.service.instance.plugin.MessageInterceptor;
import com.brilliance.eibs.el.ApacheEL;
import com.brilliance.eibs.util.ClassPathUpdater;
import com.brilliance.eibs.util.Constants;
import com.brilliance.eibs.util.StringUtil;
import org.apache.cxf.endpoint.Server;
import org.apache.cxf.interceptor.LoggingInInterceptor;
import org.apache.cxf.interceptor.LoggingOutInterceptor;
import org.apache.cxf.jaxws.JaxWsServerFactoryBean;
import org.apache.cxf.phase.Phase;
import org.apache.cxf.transport.http_jetty.JettyHTTPDestination;
import org.apache.cxf.transports.http.configuration.HTTPServerPolicy;

import java.io.File;
import java.util.HashMap;
import java.util.Map;

/**
 * webservice服务
 *
 * @author xiaoyuanzhen
 */
public class WSServer extends AbsServer {
    private static Map<String, Boolean> generatedMap = new HashMap<String, Boolean>();
    private JaxWsServerFactoryBean server;

    public WSServer(Context context, IServiceDef serviceDef, ApacheEL elParser) {
        super(context, serviceDef, elParser);
    }

    @Override
    public void run() {
        logger.info(LOG_FLAG + "starting WebService[id:" + serviceDef.getId() + "]." + LOG_FLAG);
        try {
            setReceiveTimeOut();
            // 在同一个线程内只执行一次生成
            String flg = getPropertyValue("compile");
            if (StringUtil.isEmpty(flg)) {
                flg = "true";
            }
            if (Boolean.valueOf(flg)) {
                generateBeans();
            }
            String uri = getRequiredPropertyValue(URI_SYMBOL);
            logger.info("webservice server url=" + uri);
            String clazz = getRequiredPropertyValue(CLASS_SYMBOL);
            String interfaceName = getPropertyValue("interfaceName");
            String transName = getPropertyValue("transactionName");
            server = new JaxWsServerFactoryBean();

            Class<?> z = null;
            if (Boolean.valueOf(flg)) {
                z = ClassPathUpdater.dynamicLoadClass(clazz);
            } else
                z = Class.forName(clazz);
            server.setServiceClass(z);
            server.setAddress(uri);
            context.addVariable(null, "cronExpression", serviceDef.getExpression());
            if (!StringUtil.isEmpty(interfaceName)) {
                server.getInInterceptors().add(new MessageInterceptor(context, interfaceName, transName, Phase.PRE_PROTOCOL));
                server.getInInterceptors().add(new LoggingInInterceptor());
                server.getOutInterceptors().add(new LoggingOutInterceptor());
            }
            Server endpoint = server.create();
            configTimeout(endpoint);
        } catch (Exception e) {
            recoveryServer(server);
            throw new InterfaceException("02701", e);
        }
        logger.info(LOG_FLAG + "WebService[id:" + serviceDef.getId() + "] is started." + LOG_FLAG);
    }

    private void setReceiveTimeOut() {
        String rcv_timeout = getPropertyValue(RECEIVE_TIMEOUT_SYMBOL);
        if (rcv_timeout != null) {
            int receiveTimeout = Integer.valueOf(rcv_timeout);
            if (receiveTimeout != 0) {
                this.receiveTimeout = receiveTimeout;
            }
            logger.debug("Receive timeout = " + this.receiveTimeout);
        }
    }

    /**
     * 设置返回超时
     *
     * @param service
     */
    private void configTimeout(Server server) {
        JettyHTTPDestination destination = (JettyHTTPDestination) server.getDestination();
        HTTPServerPolicy httpServerPolicy = new HTTPServerPolicy();
        httpServerPolicy.setReceiveTimeout(receiveTimeout);
        destination.setServer(httpServerPolicy);
    }

    /**
     * 回收JaxWsServerFactoryBean
     *
     * @param server
     */
    private void recoveryServer(JaxWsServerFactoryBean server) {
        if (server != null)
            server.destroy();
    }

    /**
     * 判断是否生成过class，如果没生成自动生成
     */
    public void generateBeans() {
        String beanProp = getPropertyValue("beanname");
        if (!StringUtil.isEmpty(beanProp) && !generatedMap.containsKey(beanProp)) {
            generatedMap.put(beanProp, true);
            logger.info("Creating webservice classes.");
            generateBeans(ClassPathUpdater.addClasspath(Constants.BEANS_PATH) + File.separator + beanProp, "");
            logger.info("Creating webservice classes successfully.");
        }
    }

    private void generateBeans(String configpath, String generatepath) {
        ParseConfig config = new ParseConfig();
        config.setLogger(logger);
        config.generateJavaBeanAndClass(config.parse(configpath), generatepath);
    }

    @Override
    public void close() {
        server.destroy();
        logger.info("webservice[id:" + serviceDef.getId() + "] is closed.");
    }
}
