package com.brilliance.snmp.server;

import com.brilliance.eibs.core.exception.InterfaceException;
import com.brilliance.eibs.main.Client;
import org.slf4j.Logger;
import org.snmp4j.MessageDispatcher;
import org.snmp4j.MessageDispatcherImpl;
import org.snmp4j.TransportMapping;
import org.snmp4j.agent.*;
import org.snmp4j.agent.cfg.EngineBootsCounterFile;
import org.snmp4j.agent.example.Modules;
import org.snmp4j.agent.io.DefaultMOPersistenceProvider;
import org.snmp4j.agent.io.MOInput;
import org.snmp4j.agent.io.MOInputFactory;
import org.snmp4j.agent.io.prop.PropertyMOInput;
import org.snmp4j.agent.mo.DefaultMOFactory;
import org.snmp4j.agent.mo.MOAccessImpl;
import org.snmp4j.agent.mo.MOFactory;
import org.snmp4j.agent.mo.MOScalar;
import org.snmp4j.agent.mo.util.VariableProvider;
import org.snmp4j.agent.request.Request;
import org.snmp4j.agent.request.RequestStatus;
import org.snmp4j.agent.request.SubRequest;
import org.snmp4j.agent.request.SubRequestIterator;
import org.snmp4j.log.JavaLogFactory;
import org.snmp4j.log.LogFactory;
import org.snmp4j.mp.MPv3;
import org.snmp4j.smi.*;
import org.snmp4j.transport.TransportMappings;
import org.snmp4j.util.ThreadPool;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.util.*;

/**
 * Created by gechengyang on 2014/6/21.
 */
public class SnmpEngine
        implements VariableProvider
{
  static
  {
    LogFactory.setLogFactory(new JavaLogFactory());
  }


  protected AgentConfigManager  agent;
  protected MOServer            server;
  private String                configFile;
  private File                  bootCounterFile;

  private Map<String, MOScalar> map    = new HashMap<String, MOScalar>();

  // supported MIBs
  protected Modules             modules;

  private int                   time;
  private String                oids;
  private String                interfaceName;
  private String                transactionName;
  protected Logger      logger;

  public SnmpEngine( String address, int time, String oids, String interfaceName, String transactionName, Logger      logger )
  {

    this.time = time;
    this.oids = oids;
    this.interfaceName = interfaceName;
    this.transactionName = transactionName;
    this.logger=logger;

    // ????????????
    configFile = "SampleAgent.cfg";
    bootCounterFile = new File("SampleAgent.bc");

    server = new DefaultMOServer();
    MOServer[] moServers = new MOServer[]{server};

    // ??config properties??
    InputStream configInputStream = SnmpEngine.class.getResourceAsStream("SampleAgentConfig.properties");
    // ?properties??
    final Properties props = new Properties();
    try
    {
      props.load(configInputStream);
    }
    catch (IOException ex)
    {
      ex.printStackTrace();
    }

    // ??config file
    MOInputFactory configurationFactory = new MOInputFactory()
    {
      public MOInput createMOInput()
      {
        return new PropertyMOInput(props, SnmpEngine.this);
      }
    };
    List<String> addressList = new ArrayList<String>();
    addressList.add(address);
    MessageDispatcher messageDispatcher = new MessageDispatcherImpl();
    addListenAddresses(messageDispatcher, addressList);

    agent = new AgentConfigManager(new OctetString(MPv3.createLocalEngineID()), messageDispatcher, null, moServers, ThreadPool.create(
            "SnmpServer", 3), configurationFactory, new DefaultMOPersistenceProvider(moServers, configFile), new EngineBootsCounterFile(
            bootCounterFile));

  }

  protected void addListenAddresses(MessageDispatcher md, List addresses)
  {
    for (Iterator it = addresses.iterator(); it.hasNext();)
    {
      Address address = GenericAddress.parse((String) it.next());
      TransportMapping tm = TransportMappings.getInstance().createTransportMapping(address);
      if (tm != null)
      {
        md.addTransportMapping(tm);
      }
      else
      {
        logger.warn("No transport mapping available for address '" + address + "'.");
      }
    }
  }

  public void run()
  {
    // initialize agent before registering our own modules
    agent.initialize();
    // this requires sysUpTime to be available.
    registerMIBs();
    // now continue agent setup and launch it.
    agent.run();
  }

  protected void registerMIBs()
  {
    if (modules == null)
      modules = new Modules(getFactory());
    try
    {
      // dealOid();
      modules.registerMOs(server, null);
      Timer timer = new Timer();
      timer.schedule(new TimerTask()
      {
        @Override
        public void run()
        {
          logger.info("snmp server begin to update......");
          //????oid
          for (String key : map.keySet())
          {
            server.unregister(map.get(key), null);
          }
          map.clear();
          try
          {
            dealOid();
          }
          catch (DuplicateRegistrationException e)
          {
            new InterfaceException("05002", e);
          }
        }
      }, 0, time);
    }
    catch (DuplicateRegistrationException e)
    {
      new InterfaceException("05002", e);
    }

  }

  public void dealOid()
          throws DuplicateRegistrationException
  {
    String oid[] = oids.split(",");
    for (int i = 0; i < oid.length; i++)
    {
      Map<String, Object> dataMap=(Map<String, Object>) new Client().call(interfaceName, transactionName, oid[i]).getContent();
      if(dataMap==null)
      {
        logger.info("dataMap of oids is null.......... wait for next cycle!");
        return;
      }
      //dataMap=new Client.call(); ??Map ???OID???????
      for (String key : dataMap.keySet())
      {
        Object value =dataMap.get(key);
        //???? TODO ??????????????????
        key = key + ".0";
        MOScalar myScalar=null;
        if(value instanceof Integer)
        {
          myScalar = new MOScalar(new OID(key), MOAccessImpl.ACCESS_READ_CREATE,  new  Integer32(Integer.parseInt(String.valueOf(value))));
        }
        else
        {
          myScalar = new MOScalar(new OID(key), MOAccessImpl.ACCESS_READ_CREATE, new OctetString(String.valueOf(value)));
        }

        map.put(key, myScalar);
        server.register(myScalar, null);
      }
    }
  }

  public Variable getVariable(String name)
  {

    OID oid;
    OctetString context = null;
    int pos = name.indexOf(':');
    if (pos >= 0)
    {
      context = new OctetString(name.substring(0, pos));
      oid = new OID(name.substring(pos + 1, name.length()));
    }
    else
    {
      oid = new OID(name);
    }

    final DefaultMOContextScope scope = new DefaultMOContextScope(context, oid, true, oid, true);
    MOQuery query = new DefaultMOQuery(scope, false, this);
    ManagedObject mo = server.lookup(query);

    if (mo != null)
    {
      final VariableBinding vb = new VariableBinding(oid);
      final RequestStatus status = new RequestStatus();
      SubRequest req = new SubRequest()
      {
        private boolean completed;
        private MOQuery query;

        public boolean hasError()
        {
          return false;
        }

        public void setErrorStatus(int errorStatus)
        {
          status.setErrorStatus(errorStatus);
        }

        public int getErrorStatus()
        {
          return status.getErrorStatus();
        }

        public RequestStatus getStatus()
        {
          return status;
        }

        public MOScope getScope()
        {
          return scope;
        }

        public VariableBinding getVariableBinding()
        {
          return vb;
        }

        public Request getRequest()
        {
          return null;
        }

        public Object getUndoValue()
        {
          return null;
        }

        public void setUndoValue(Object undoInformation)
        {}

        public void completed()
        {
          completed = true;
        }

        public boolean isComplete()
        {
          return completed;
        }

        public void setTargetMO(ManagedObject managedObject)
        {}

        public ManagedObject getTargetMO()
        {
          return null;
        }

        public int getIndex()
        {
          return 0;
        }

        public void setQuery(MOQuery query)
        {
          this.query = query;
        }

        public MOQuery getQuery()
        {
          return query;
        }

        public SubRequestIterator repetitions()
        {
          return null;
        }

        public void updateNextRepetition()
        {}

        public Object getUserObject()
        {
          return null;
        }

        public void setUserObject(Object userObject)
        {

        }

      };
      mo.get(req);
      return vb.getVariable();
    }
    return null;
  }

  protected MOFactory getFactory()
  {
    return DefaultMOFactory.getInstance();
  }

  public static void main(String[] args)
  {

    String address = "udp:127.0.0.1/166";

    SnmpEngine sampleAgent = new SnmpEngine(address,5000,"1.3.4,2.4.5","test","test",null);
    // Add all available security protocols (e.g. SHA,MD5,DES,AES,3DES,..)
    //SecurityProtocols.getInstance().addDefaultProtocols();

    sampleAgent.run();

  }

}